@@ -12,7 +12,7 @@ fn test_process() {
     let mut s = sysinfo::System::new();
     assert_eq!(s.processes().len(), 0);
     s.refresh_processes();
-    if !sysinfo::System::IS_SUPPORTED {
+    if !sysinfo::System::IS_SUPPORTED || cfg!(feature = "apple-sandbox") {
         return;
     }
     assert!(!s.processes().is_empty());
@@ -23,12 +23,42 @@ fn test_process() {
         .any(|p| !p.exe().to_str().unwrap_or("").is_empty()));
 }
 
+#[test]
+#[cfg(not(windows))]
+fn test_process_cwd() {
+    let p = std::process::Command::new("sleep")
+        .arg("3")
+        .spawn()
+        .unwrap();
+    let pid = p.id() as sysinfo::Pid;
+
+    let mut s = sysinfo::System::new();
+    s.refresh_processes();
+
+    if !sysinfo::System::IS_SUPPORTED || cfg!(feature = "apple-sandbox") {
+        return;
+    }
+
+    let processes = s.processes();
+    let p = processes.get(&pid);
+
+    if let Some(p) = p {
+        assert_eq!(p.pid(), pid);
+        assert_eq!(
+            p.cwd().to_str().unwrap(),
+            std::env::current_dir().unwrap().to_str().unwrap()
+        );
+    } else {
+        panic!("Process not found");
+    }
+}
+
 #[test]
 fn test_process_refresh() {
     let mut s = sysinfo::System::new();
     assert_eq!(s.processes().len(), 0);
 
-    if !sysinfo::System::IS_SUPPORTED {
+    if !sysinfo::System::IS_SUPPORTED || cfg!(feature = "apple-sandbox") {
         return;
     }
     s.refresh_process(sysinfo::get_current_pid().expect("failed to get current pid"));
@@ -60,7 +90,7 @@ fn test_get_cmd_line() {
 #[test]
 #[cfg(not(windows))]
 fn test_get_cmd_line() {
-    if sysinfo::System::IS_SUPPORTED {
+    if sysinfo::System::IS_SUPPORTED && !cfg!(feature = "apple-sandbox") {
         unix_like_cmd();
     }
 }
@@ -84,6 +114,38 @@ fn unix_like_cmd() {
     }
 }
 
+#[test]
+#[cfg(not(windows))]
+fn test_environ() {
+    if !sysinfo::System::IS_SUPPORTED {
+        return;
+    }
+
+    let mut p = std::process::Command::new("sleep")
+        .arg("3")
+        .stdout(std::process::Stdio::null())
+        .env("FOO", "BAR")
+        .env("OTHER", "VALUE")
+        .spawn()
+        .unwrap();
+    let pid = p.id() as sysinfo::Pid;
+    std::thread::sleep(std::time::Duration::from_millis(250));
+    let mut s = sysinfo::System::new();
+    s.refresh_processes();
+    p.kill().unwrap();
+
+    let processes = s.processes();
+    let p = processes.get(&pid);
+
+    if let Some(p) = p {
+        assert_eq!(p.pid(), pid);
+        assert!(p.environ().iter().any(|e| e == "FOO=BAR"));
+        assert!(p.environ().iter().any(|e| e == "OTHER=VALUE"));
+    } else {
+        panic!("No process found!")
+    }
+}
+
 #[test]
 fn test_process_disk_usage() {
     use std::fs;
@@ -91,7 +153,7 @@ fn test_process_disk_usage() {
     use std::io::prelude::*;
     use sysinfo::{get_current_pid, ProcessExt, SystemExt};
 
-    if !sysinfo::System::IS_SUPPORTED {
+    if !sysinfo::System::IS_SUPPORTED || cfg!(feature = "apple-sandbox") {
         return;
     }
 
@@ -126,7 +188,7 @@ fn cpu_usage_is_not_nan() {
     let mut system = sysinfo::System::new();
     system.refresh_processes();
 
-    if !sysinfo::System::IS_SUPPORTED {
+    if !sysinfo::System::IS_SUPPORTED || cfg!(feature = "apple-sandbox") {
         return;
     }
 
